import random
import re
from typing import List, Literal, Type

from nonebot import get_driver, on_command, on_message
from nonebot.adapters.onebot.v11 import Message, MessageEvent, MessageSegment
from nonebot.matcher import Matcher
from nonebot.params import ArgPlainText, CommandArg
from nonebot.typing import T_State

from .config import config
from .data_source import (
    bnhhsh_trans,
    not_translate,
    parse_yinglish_param,
    unvcode_convert,
    yinglish_trans,
)

nickname = n[0] if (n := list(get_driver().config.nickname)) else "我"


def get_command_matcher(commands: List[str], *args, **kwargs) -> Type[Matcher]:
    return on_command(commands[0], *args, aliases=set(commands[1:]), **kwargs)


matcher_unvcode = get_command_matcher(config.rimofun_unvcode_commands)
matcher_bnhhsh = get_command_matcher(config.rimofun_bnhhsh_commands)
matcher_yinglish = get_command_matcher(config.rimofun_yinglish_commands)
matcher_translate = get_command_matcher(config.rimofun_translator_commands)


@matcher_unvcode.handle()
@matcher_bnhhsh.handle()
@matcher_yinglish.handle()
@matcher_translate.handle()
async def _(matcher: Matcher, arg: Message = CommandArg()):
    if arg.extract_plain_text().strip():
        matcher.set_arg("arg", arg)


@matcher_unvcode.got("arg", f"请问您要让{nickname}帮忙反屏蔽什么文本呢？")
@matcher_bnhhsh.got("arg", f"请问您要让{nickname}帮你解释一下什么看不懂的缩写呢？")
@matcher_yinglish.got(
    "arg",
    (
        f"{nickname}要让您接下来发出来的话变得涩涩哦♡~快请您发送消息吧！\n"
        "您还可以在消息后面跟上空格，再加上一个百分比，就可以控制生成文本的涩涩程度！"
    ),
)
@matcher_translate.got("arg", f"要让{nickname}帮您翻译一下什么语句呢？")
async def _(matcher: Matcher, arg: str = ArgPlainText("arg")):
    if not arg.strip():
        await matcher.reject(f"hmm……{nickname}没有在您刚刚发的消息里看到文本呢……请您重新发一下吧qwq")
    matcher.skip()


@matcher_unvcode.handle()
async def _(matcher: Matcher, arg: str = ArgPlainText("arg")):
    result = unvcode_convert(arg.strip())
    await matcher.finish(f"{nickname}已经帮您处理好啦！请看看下面的结果~\n{result}")


@matcher_bnhhsh.handle()
async def _(matcher: Matcher, arg: str = ArgPlainText("arg")):
    result = bnhhsh_trans(arg)
    if not result:
        await matcher.finish(f"{nickname}没有在刚刚的文本中找到英文呢……如果您还需要帮助，请重新询问我吧~")

    message = "\n".join([f"{k}：{v}" for k, v in result.items()])
    await matcher.finish(f"就让学识渊博的{nickname}来帮您解释清楚吧(/≧▽≦)/\n{message}")


@matcher_yinglish.handle()
async def _(matcher: Matcher, arg: str = ArgPlainText("arg")):
    try:
        txt, percent = parse_yinglish_param(arg.strip())
    except ValueError:
        await matcher.finish(
            f"{nickname}发现您输入的涩涩程度百分比有问题哦quq\n这个百分比需要大于 0%，同时小于等于 100% 呢……",
        )

    result = yinglish_trans(txt, percent)
    await matcher.finish(f"收到命令，{nickname}马上就让您的话语变得涩涩起来！！\n{result}")


@matcher_translate.handle()
async def _(matcher: Matcher, arg: str = ArgPlainText("arg")):
    result = not_translate(arg)
    await matcher.finish(f"请您看看{nickname}的翻译厉不厉害！嘿嘿~\n{result}")


async def trigger_rule(event: MessageEvent, state: T_State):
    msg_txt = event.get_plaintext().strip()
    if not msg_txt:
        return False

    is_english = re.fullmatch(r"^[a-zA-Z0-9@$!%*?&#^-_.+\s]+$", msg_txt)
    random_val = random.random()

    types: List[Literal["bnhhsh", "yinglish", "translator"]] = []

    if (
        is_english
        and config.rimofun_bnhhsh_reply_chance
        and random_val < config.rimofun_bnhhsh_reply_chance
    ):
        types.append("bnhhsh")

    if (
        config.rimofun_yinglish_reply_chance
        and random_val < config.rimofun_yinglish_reply_chance
    ):
        types.append("yinglish")

    if (
        is_english
        and config.rimofun_translator_reply_chance
        and random_val < config.rimofun_translator_reply_chance
    ):
        types.append("translator")

    if types:
        state["type"] = random.choice(types)
        return True

    return False


matcher_initiative = on_message(
    rule=trigger_rule,
    block=False,
    priority=config.rimofun_initiative_priority,
)


@matcher_initiative.handle()
async def _(matcher: Matcher, event: MessageEvent, state: T_State):
    msg_txt = event.get_plaintext().strip()
    trigger_type: Literal["bnhhsh", "yinglish", "translator"] = state["type"]
    reply_seg = MessageSegment.reply(event.message_id)

    if trigger_type == "bnhhsh":
        result = bnhhsh_trans(msg_txt)
        reply = "\n".join([f"{k}：{v}" for k, v in result.items()])
        await matcher.finish(reply_seg + f"让{nickname}解释一下消息里出现的神秘字母吧~\n{reply}")

    if trigger_type == "yinglish":
        reply = yinglish_trans(msg_txt)
        await matcher.finish(reply_seg + f"让{nickname}来把你说的话变得更涩一点~\n{reply}")

    if trigger_type == "translator":
        reply = not_translate(msg_txt)
        await matcher.finish(reply_seg + f"让{nickname}把这条消息翻译一下吧！\n{reply}")



import re
from typing import Dict, List, Tuple

from nonebot import logger
from unvcode import unvcode
from yinglish import chs2yin

from .bnhhsh.bnhhsh import dp
from .config import config
from .not_translator.not_translator import translate


def char_to_hex(s):
    return f"0x{s.encode('utf-8').hex()}"


def unvcode_convert(txt: str) -> str:
    result, mse = unvcode(txt, mse=config.rimofun_unvcode_mse)
    change_dict: Dict[str, Tuple[str, float]] = {
        txt[i]: (result[i], x) for i, x in enumerate(mse) if (x is not None)
    }

    for ori, (res, n) in change_dict.items():
        logger.debug(
            "unvcode: "
            f"替换 {ori}({char_to_hex(ori)}) -> {res}({char_to_hex(res)}) "
            f"({(1-n) * 100:.2f}% 相似)",
        )

    return result


def bnhhsh_trans(txt: str) -> Dict[str, str]:
    words: List[str] = [x.lower() for x in re.findall(r"[a-zA-Z-_]+", txt)]
    will_trans = []
    for it in words:
        if it not in will_trans:
            will_trans.append(it)

    translated = [dp(x) for x in will_trans]

    if config.rimofun_bnhhsh_unvcode:
        translated = [unvcode_convert(x) for x in translated]

    return {will_trans[i]: x for i, x in enumerate(translated)}


def parse_yinglish_param(params: str) -> Tuple[str, float]:
    if params.endswith("%"):
        param_li = params.split(" ")

        txt = " ".join(param_li[:-1])
        percent = param_li[-1][:-1]

        if re.fullmatch(r"^([0-9]+)(.([0-9]+))?$", percent):
            percent_num = float(percent) / 100
            if not 0 < percent_num <= 1:
                raise ValueError

            return txt, percent_num

    return params, config.rimofun_yinglish_yingness


def yinglish_trans(txt: str, yingness: float = config.rimofun_yinglish_yingness) -> str:
    result = chs2yin(txt, yingness)

    if config.rimofun_yinglish_unvcode:
        return unvcode_convert(result)

    return result


def not_translate(txt: str) -> str:
    return translate(txt)



import json
from functools import lru_cache

import Levenshtein


ld = lru_cache(maxsize=None)(Levenshtein.distance)


def 代价(对):
    return sum([ld(a, b) for a, b in 对])/len(对)


def load_dict(path) -> dict:
    d = {}
    with open(path, encoding='utf8') as f:
        for i in f:
            i = i.strip()
            a, b = i.split('\t')
            b = b.split(',')[0]
            assert b[0] == b[-1] == '/'
            b = b[1:-1]
            d[a] = b
    return d


英文ipa = load_dict('../not_translator/ipa_data/en_US.txt')
中文ipa = load_dict('../not_translator/ipa_data/zh_hans.txt')

英文ipa = {k.lower(): v for k, v in 英文ipa.items()}
中文ipa = {k: v.replace('˥˥', '').replace('˧˥', '').replace('˨˩˦', '').replace('˥˩', '').replace('˧', '') for k, v in 中文ipa.items() if len(k) == 1}


with open('data/名字.json', encoding='utf8') as f:
    名字对应 = json.load(f)

对 = []

for k, v in 名字对应.items():
    k = k.lower()
    真读 = 英文ipa.get(k)
    if not 真读:
        continue
    假读 = ''.join([中文ipa[x] for x in v])
    真读 = f'「{真读}」'
    假读 = f'「{假读}」'
    对.append((真读, 假读))



import json
import pickle
import tempfile
from pathlib import Path

here = Path(__file__).parent


def 破处():
    def 缩(s):
        from pypinyin import pinyin, Style
        q = pinyin(s, style=Style.FIRST_LETTER)
        return ''.join([x[0].lower() for x in q])
    def 丢(词桶, 词表, 痛苦):
        for i in 词表:
            k = 缩(i)
            w = 词桶.setdefault(len(i), {})
            if y:=w.get(k):
                if y[1] > 痛苦:
                    w[k] = [f'{i}', 痛苦]
            else:
                w[k] = [f'{i}', 痛苦]
    with open(here/'data/色情词库.json', encoding='utf8') as f:
        色情词库 = json.load(f)    
    with open(here/'data/色情词库_数据增强.json', encoding='utf8') as f:
        色情词库_数据增强 = json.load(f)
    with open(here/'data/莉沫词库.json', encoding='utf8') as f:
        莉沫词库 = json.load(f)
    with open(here/'data/常用汉字.json', encoding='utf8') as f:
        常用汉字 = json.load(f)
    with open(here/'data/现代汉语常用词表.json', encoding='utf8') as f:
        现代汉语常用词表 = json.load(f)
    词桶 = {
        1: {'i': ['爱', 0.1], 'u': ['幼', 0.1]}
    }
    丢(词桶, 色情词库, 0.001)
    丢(词桶, 莉沫词库, 0.01)
    丢(词桶, 色情词库_数据增强, 0.1)
    丢(词桶, 常用汉字, 0.11)
    丢(词桶, 现代汉语常用词表, 0.2)
    n = max(词桶)
    for i in range(1, n+1):
        词桶.setdefault(i, {})
    with open(Path(tempfile.gettempdir())/'bnhhsh词桶v1.2.2.pkl', 'wb') as f:
        pickle.dump(词桶, f)

q = Path(Path(tempfile.gettempdir())/'bnhhsh词桶v1.2.2.pkl')
if not q.is_file():
    破处()
with open(q, 'rb') as f:
    词桶 = pickle.load(f)

n = max(词桶)

def yndp(target):
    代价 = {-1: 0}
    记录 = {-1: []}
    for x in range(len(target)):
        代价[x] = 2**32
        for k in range(n, 0, -1):
            s = x-k+1
            if s < 0:
                continue
            if c:=词桶[k].get(target[s:x+1]):
                词, 痛苦 = c
                if 代价[x-k]+痛苦 < 代价[x]:
                    代价[x] = 代价[x-k]+痛苦
                    记录[x] = 记录[x-k].copy()
                    记录[x].append((s, x+1, 词))
        if 代价[x-1]+1 < 代价[x]:
            代价[x] = 代价[x-1]+1
            记录[x] = 记录[x-1].copy()
    target = [*target]
    for a, b, c in 记录[len(target)-1][::-1]:
        target[a:b] = c
    return ''.join(target), 代价[len(target)-1]

def dp(target):
    return yndp(target)[0]



import setuptools


setuptools.setup(
    name='bnhhsh',
    version='1.2.2',
    author='RimoChan',
    author_email='the@librian.net',
    description='bnhhsh',
    url='https://github.com/RimoChan/bnhhsh',
    packages=['bnhhsh'],
    package_data={
        'bnhhsh': ['data/色情词库.json', 'data/色情词库_数据增强.json', 'data/莉沫词库.json', 'data/常用汉字.json', 'data/现代汉语常用词表.json'],
    },
    classifiers=[
        'Programming Language :: Python :: 3',
        'Operating System :: OS Independent',
    ],
    install_requires=[
        'pypinyin>=0.42.0',
    ],
    python_requires='>=3.8',
)



import time
import json
from functools import lru_cache
import random
from typing import List, Optional, Tuple

from 数据 import 对, 代价
from collections import Counter

import Levenshtein

ld = lru_cache(maxsize=None)(Levenshtein.distance)


@lru_cache(maxsize=None)
def 破碎(s, l) -> Tuple[str, ...]:
    a = []
    for q in range(0, len(s)+1):
        for w in range(q+1, len(s)+1):
            if w-q <= l:
                a.append(s[q:w])
    return tuple(a)


def 引力(a, b) -> Optional[Tuple[str, str]]:
    if a == b:
        return None
    l = ld(a, b)
    aa = 破碎(a, 3)
    bb = 破碎(b, 3) + (' ',)
    x = random.choice(aa)
    y = random.choice(bb)
    for _ in range(10):
        if ld(a.replace(x, y), b) < l:
            return (x, y)
    return None


def 收(对) -> List:
    a = [引力(a, b) for a, b in 对]
    return [i for i in a if i]

print(对)
exit()


超录 = []
当前代价 = 代价(对)
否决 = set()
while True:
    c = Counter()
    while True:
        s = [i for i in 收(对) if i not in 否决]
        c.update(s)
        (x, y), t = Counter(c).most_common(1)[0]
        if t > 5:
            break
    新对 = [(a.replace(x, y), b) for a, b in 对]
    新代价 = 代价(新对)
    if 新代价 >= 当前代价:
        否决.add((x, y))
        print(f'否决{x}-{y}, 当前代价={当前代价}, 新代价={新代价}')
    else:
        对 = 新对
        当前代价 = 代价(对)
        否决 = set()
        print(f'通过{x}-{y}，当前代价={当前代价}')
        超录.append((x, y))
        with open(f'超录/{str(新代价)[:6]+"_"+str(random.randint(11, 99))}.json', 'w', encoding='utf8') as f:
            f.write(json.dumps(超录, ensure_ascii=False))



from typing import List

from nonebot import get_driver
from pydantic import BaseModel


class ConfigModel(BaseModel):
    # unvcode
    rimofun_unvcode_mse: float = 0.2
    """unvcode 的全局字符串不同阈值，比如 0.2 代表匹配 80% 相同的字符"""

    rimofun_unvcode_commands: List[str] = ["unvcode", "幼女码"]
    """手动触发 bnhhsh 的命令"""

    # bnhhsh
    rimofun_bnhhsh_unvcode: bool = True
    """bnhhsh 是否使用 unvcode 反和谐"""

    rimofun_bnhhsh_commands: List[str] = ["bnhhsh", "不能好好说话"]
    """手动触发 bnhhsh 的命令"""

    rimofun_bnhhsh_reply_chance: float = 0.0
    """bnhhsh 自动回复纯英文消息的触发概率，值应在 0~1 之间"""

    # yinglish
    rimofun_yinglish_unvcode: bool = False
    """yinglish 是否使用 unvcode 反和谐"""

    rimofun_yinglish_commands: List[str] = ["yinglish", "淫语"]
    """手动触发 yinglish 的命令"""

    rimofun_yinglish_reply_chance: float = 0.0
    """yinglish 自动回复任意文本消息的触发概率，值应在 0~1 之间"""

    rimofun_yinglish_yingness: float = 0.7
    """yinglish 默认淫乱度，值应在 0~1 之间"""

    # not translator
    rimofun_translator_commands: List[str] = ["not translator", "不会翻译"]
    """手动触发 not-translator 的命令"""

    rimofun_translator_reply_chance: float = 0.0
    """translator 自动回复纯英文消息的触发概率，值应在 0~1 之间"""

    # other
    rimofun_initiative_priority: int = 99
    """自动回复 matcher 的优先级"""


config: ConfigModel = ConfigModel.parse_obj(get_driver().config.dict())



from nonebot.plugin import PluginMetadata

from . import __main__ as __main__
from .config import ConfigModel, config

__version__ = "0.1.1"
__plugin_meta__ = PluginMetadata(
    name="RimoFun",
    description="用 RimoChan 开发的工具做的一个有趣的插件",
    usage=(
        "指令列表：\n"
        "注：以下指令后面带不带参数都可以，如果没带参数，Bot 将会询问你\n"
        " \n"
        "▶ unvcode\n"
        "  ▷ 介绍：反和谐超级武器，幼女 Code！\n"
        f"  ▷ 指令：{'；'.join(config.rimofun_unvcode_commands)}\n"
        " \n"
        "▶ bnhhsh\n"
        "  ▷ 介绍：不对劲的拼音首字母缩写翻译工具！\n"
        f"  ▷ 指令：{'；'.join(config.rimofun_bnhhsh_commands)}\n"
        " \n"
        "▶ yinglish\n"
        "  ▷ 介绍：淫语翻译机！\n"
        f"  ▷ 指令：{'；'.join(config.rimofun_yinglish_commands)}\n"
        " \n"
        "▶ not translator\n"
        "  ▷ 介绍：不会翻译机！\n"
        f"  ▷ 指令：{'；'.join(config.rimofun_translator_commands)}"
    ),
    homepage="https://github.com/lgc-NB2Dev/nonebot-plugin-rimofun",
    type="application",
    config=ConfigModel,
    supported_adapters=["~onebot.v11"],
    extra={"License": "MIT", "Author": "student_2333"},
)



import re
import json
import math
from pathlib import Path
from collections import Counter


here = Path(__file__).parent

with open(here/'摸型.json', 'r', encoding='utf8') as f:
    模型 = json.load(f)
with open(here/'常用汉字.json', encoding='utf8') as f:
    常用汉字 = {*json.load(f)}


def load_dict(path) -> dict:
    d = {}
    with open(path, encoding='utf8') as f:
        for i in f:
            i = i.strip()
            a, b = i.split('\t')
            b = b.split(',')[0]
            assert b[0] == b[-1] == '/'
            b = b[1:-1]
            d[a] = b
    return d


英文ipa = load_dict(here/'ipa_data/en_US.txt')
中文ipa = load_dict(here/'ipa_data/zh_hans.txt')

英文ipa = {k.lower(): v for k, v in 英文ipa.items()}
中文ipa = {k: v.replace('˥˥', '').replace('˧˥', '').replace('˨˩˦', '').replace('˥˩', '').replace('˧', '').replace(' ', '') for k, v in 中文ipa.items()}


def 转移(s, 风):
    l = []
    源a = f'「{英文ipa[s]}」'
    for f in 风:
        a = 源a
        for x, y in f:
            a = a.replace(x, y)
        l.append(a.replace('「', '').replace('」', ''))
    return l


反向索引 = {}
for k, v in 中文ipa.items():
    if k in 常用汉字:
        反向索引.setdefault(v, k)
for k, v in 中文ipa.items():
    反向索引.setdefault(v, k)
反向索引['ða'] = 反向索引['la']  # 手动修复
k = max([len(i) for i in 反向索引])


def dp(s: str):
    n = len(s)
    代价 = [0] + [math.inf] * n
    记录 = [''] + ['?'] * n
    for i in range(n+1):
        for j in range(max(0, i-k), i):
            if s[j:i] in 反向索引:
                if 代价[j] < 代价[i]:
                    代价[i] = 代价[j]
                    记录[i] = 记录[j] + 反向索引[s[j:i]]
            else:
                if 代价[j]+(i-j) < 代价[i]:
                    代价[i] = min(代价[i], 代价[j]+(i-j))
                    记录[i] = 记录[j] + s[j:i]
    return 记录[-1], 代价[-1]


def 真移(s):
    s = s.lower()
    if s not in 英文ipa:
        return s
    l = [i[0] for i in Counter(转移(s, 模型)).most_common(9)]
    for x in l:
        z, w = dp(x)
        if w == 0:
            return z
    return dp(l[0])[0]


def translate(s: str) -> str:
    return re.sub('[a-zA-Z]+', lambda x: 真移(x.group()), s)
